from yarl import URL
from obt import dep, path, pathtools, command

###############################################################################

class dsp(dep.StdProvider):
  VERSION = "v1.5.0"
  name = "dsp"
  def __init__(self):
    build_dir = path.builds()/dsp.name
    super().__init__(dsp.name)
    
    self.dest_header_path = path.includes()/"dsp"
    
    self._builder = self.createBuilder(dep.CustomBuilder)
    self._builder._cleanOnClean = False
    
    def before_build():
      print("before_build")
      pathtools.ensureDirectoryExists(self.dest_header_path)
      items = pathtools.patglob(self.source_root, "*.h")
      copy_commands = []
      for item in items:
        print(item)
        cmd = command.Command([
          "cp",str(item), str(self.dest_header_path)+"/"
          ])
        copy_commands.append(cmd)
      self._builder._cleanbuildcommands = copy_commands
      self._builder._incrbuildcommands = copy_commands
      print("copy_commands: %s" % copy_commands)

    self._builder._invokeBeforeBuild = before_build

    
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=dsp.name,
                             repospec="Signalsmith-Audio/dsp",
                             revision=dsp.VERSION,
                             recursive=False)

  #######################################################################

  def areRequiredSourceFilesPresent(self):
    print(self.source_root)
    return (self.source_root/"README.md").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"dsp"/"fft.h").exists()



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from obt import dep, host, path, pathtools
from obt.command import Command
###############################################################################

class ispctexc(dep.StdProvider):
  name = "ispctexc"
  def __init__(self):
    super().__init__(ispctexc.name)
    self.declareDep("ispc")
    self.build_dest = self.source_root/"build"
    self._archlist = ["x86_64"]

  ########################################################################
  @property
  def _fetcher(self):
    fetcher = dep.GithubFetcher(name=ispctexc.name,
                                repospec="tweakoz/ISPCTextureCompressor",
                                revision="master",
                                recursive=False)
    fetcher._cache=False
    return fetcher

  ########################################################################

  def build(self):

    dep.require("ispc")

    #########################################
    # fetch source
    #########################################

    if not self.source_root.exists():
      self._fetcher.fetch(self.source_root)

    #########################################
    # build
    #########################################

    self.source_root.chdir()
    r = Command(["make","-f","Makefile.linux"]).exec()
    OK = (r==0)

    if OK:
      OK = self.install()

    return OK

  def install(self):
    sonam = "libispc_texcomp.so"
    hdrnam = "ispc_texcomp.h"
    if host.IsOsx:
      cmd = ["install_name_tool",
             "-id","@rpath/libispc_texcomp.so",
             self.build_dest/sonam]
      Command(cmd).exec()
    pathtools.copyfile(self.build_dest/sonam,path.stage()/"lib"/sonam)
    pathtools.copyfile(self.source_root/"ispc_texcomp"/hdrnam,path.stage()/"include"/hdrnam)
    return True

  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"Makefile.linux").exists()

  def areRequiredBinaryFilesPresent(self):
    return (path.libs()/"libispc_texcomp.so").exists()

 



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from obt import dep, path

###############################################################################

class libsocket(dep.StdProvider):
  name = "libsocket"
  def __init__(self):
    super().__init__(libsocket.name)
    self.declareDep("cmake")
    self._builder = self.createBuilder(dep.CMakeBuilder)
    self._builder.requires(["cmake"])
    self._debug = True
    self._builder._debug = True
  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=libsocket.name,
                             repospec="tweakoz/libsocket",
                             revision="master",
                             recursive=False,
                             #debug=True,
                             )
  ########################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"package.xml").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"libsocket"/"epoll.hpp").exists()



#!/usr/bin/env python3
###############################################################################
# Orkid Build System
# Copyright 2010-2018, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

import argparse, sys, os, tempfile
from string import Template
from obt import deco, path
from obt.eda.xilinx import vivado

deco = deco.Deco()

###############################################################################

epilog = deco.orange("Example (vivado help): %s --batch -- -help"%sys.argv[0])
parser = argparse.ArgumentParser(description='Launch command in EDA Docker Containers',
                                 epilog=epilog)
parser.add_argument('--prim', help=deco.yellow('primitive type'))
parser.add_argument('--partid', help=deco.yellow('fpga partid'))

_args = vars(parser.parse_args())

if len(sys.argv)==1:
    print(parser.format_usage())
    sys.exit(1)

###############################################################################

partid = _args["partid"]
assert(partid!=None)

###############################################################################
#set site [get_sites GTHE4_CHANNEL_X0Y19]
#report_property -all [get_package_pins A8]
#report_property -all $site
#get_property CLOCK_REGION $site
#get_property PIN_FUNC [get_package_pins A7] #MGTHTXN3_227
#get_site_pins -of $site
#get_bels -of $site # GTHE4_CHANNEL_X0Y22/GTHE4_CHANNEL GTHE4_CHANNEL_X0Y22/IPAD1
#                     GTHE4_CHANNEL_X0Y22/IPAD2 GTHE4_CHANNEL_X0Y22/OPAD1 GTHE4_CHANNEL_X0Y22/OPAD2

#get_sites [get_package_pins -filter { PIN_FUNC =~ *MGTHTXP3_227* }] # GTHE4_CHANNEL_X0Y19

#get_bel_pins -of $site
#get_tiles -of_objects $site # GTH_QUAD_RIGHT_X69Y300

topstr = "module top(); endmodule"

TEMPLATE = Template("""
puts "Getting Sites"
create_project -part $PARTNAME -in_memory -verbose
read_verilog [glob /tmp/build/*.v]
synth_design -rtl -top top
set items [ get_sites -regexp "GTHE4_.*"]
#set items [ get_sites]
foreach item $$items {
  puts "site<$$item>"
  set site [get_sites $$item]
  set pins [ get_site_pins ‐of $$site ]
  puts "site<$$item> pins<$$pins>"
}
""")
tclstr = TEMPLATE.substitute(PARTNAME=partid)

###############################################################################

with tempfile.TemporaryDirectory() as tempdir:
  tempdirname = path.Path(tempdir)
  vctx = vivado.Context(hostdir=tempdirname,FPGAPART=partid)
  tclhostfilename = vctx.hostdir/"getsites.tcl"
  tclcontfilename = vctx.containerdir/"getsites.tcl"
  verhostfilename = vctx.hostdir/"top.v"
  vercontfilename = vctx.containerdir/"top.v"
  with open(tclhostfilename,"wt") as f:
    f.write(tclstr)
  with open(verhostfilename,"wt") as f:
    f.write(topstr)
  rval = vctx.run_tclscript(tclcontfilename)



###############################################################################
# Orkid Build System
# Copyright 2010-2023, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from obt import dep, log, path, host, env, osrelease
from obt.command import Command
###############################################################################
SRC_VERSION = "v2021.11.0"
NAME = "tbb"
class _tbb_from_source(dep.StdProvider):
  def __init__(self):
    super().__init__(NAME)

    self._builder = self.createBuilder(dep.CMakeBuilder)
    self._builder._cmakeenv = {
      "BUILD_SHARED_LIBS": "ON",
    }
    
    desc = osrelease.descriptor()

    use_gcc_11 = desc.version_id == "23.10"
    use_gcc_11 = use_gcc_11 or (desc.version_id == "24.04")

    if use_gcc_11 and host.IsLinux:
      self._builder._cmakeenv["CMAKE_CXX_COMPILER"] = "g++-11"
      self._builder._cmakeenv["CMAKE_C_COMPILER"] = "gcc-11"

    #self._builder = self.createBuilder(dep.CustomBuilder)
    #build_dir = path.subspace_build_dir/"tbb"
    #env = {
     # "PREFIX": path.stage()
    #}

    #make_clean_command = Command([
    #  "make",
    #  "-j",host.NumCores
    #  ],
    #  working_dir=build_dir,
    #  environment=env)

    #make_incr_command = Command([
    #  "make",
    #  "-j",host.NumCores],
    #  working_dir=build_dir,
    #  environment=env)

    #self._builder._cleanbuildcommands += [make_clean_command]
    #self._builder._incrbuildcommands += [make_incr_command]
    #self._tbb_dir = build_dir

  def env_init(self):
    log.marker("registering TBB(%s) SDK"%SRC_VERSION)
    #env.set("TBB_ROOT",self._tbb_dir)
    #env.set("TBB_INCLUDE",self._tbb_dir/"include")
    #env.set("TBB_LIBRARY_RELEASE" = $TBB_INSTALL_DIR/build/RELEASE_FOLDER
    #env.set("TBB_LIBRARY_DEBUG" = $TBB_INSTALL_DIR/build/DEBUG_FOLDER

  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=NAME,
                             repospec="oneapi-src/oneTBB",
                             revision=SRC_VERSION,
                             recursive=False)
###############################################################################
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()

  def areRequiredBinaryFilesPresent(self):
    return path.decorate_obt_lib("openvdb").exists()
###############################################################################
class _tbb_from_homebrew(dep.HomebrewProvider):
  def __init__(self):
    super().__init__("tbb","tbb")
  def env_init(self):
    log.marker("registering TBB(homebrew) SDK")
###############################################################################
source = dep.switch(linux=_tbb_from_source,macos=_tbb_from_homebrew)
###############################################################################
class tbb(source):
  def __init__(self):
    super().__init__()
  def env_init(self):
    super().env_init()
###############################################################################




###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################
from obt import dep, log, path, host
from obt.command import Command
###############################################################################
class icestorm(dep.StdProvider):
  name = "icestorm"
  VERSION = "83b8ef947f77723f602b706eac16281e37de278c"
  def __init__(self):
    super().__init__(icestorm.name)
    ###########################################
    self._builder = self.createBuilder(dep.CustomBuilder)
    self._builder._cleanOnClean = False
    
    build_dir = path.builds()/icestorm.name

    env = {
    	"PREFIX": path.stage()
    }

    make_clean_command = Command([
    	"make",
    	"-j",host.NumCores,
    	"clean","install"
    	],
    	working_dir=build_dir,
    	environment=env)

    make_incr_command = Command([
    	"make",
    	"-j",host.NumCores,
    	"install"],
    	working_dir=build_dir,
    	environment=env)

    self._builder._cleanbuildcommands += [make_clean_command]
    self._builder._incrbuildcommands += [make_incr_command]

  ########################################################################
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=icestorm.name,
                             repospec="YosysHQ/icestorm",
                             revision=icestorm.VERSION,
                             recursive=False)

  #######################################################################



# persistent configuration

import json

def load(configpath):
    inpstr = ""
    load_dict = dict()
    ########################
    try:
        with open(str(configpath),"r") as f:
            inpstr = f.read()
            load_dict = json.loads(inpstr)
    except:
        pass
    ########################
    return load_dict


def merge(configpath,defaults):
    inpstr = ""
    overlay_dict = dict()
    ########################
    try:
        with open(str(configpath),"r") as f:
            inpstr = f.read()
            overlay_dict = json.loads(inpstr)
    except:
        pass
    ########################
    config_dict = dict(defaults, **overlay_dict)
    with open(str(configpath),"w") as f:
        f.write(json.dumps(config_dict, sort_keys=True, indent=4 ))
    return config_dict



###############################################################################
# Orkid Build System
# Copyright 2010-2020, Michael T. Mayers
# email: michael@tweakoz.com
# The Orkid Build System is published under the GPL 2.0 license
# see http://www.gnu.org/licenses/gpl-2.0.html
###############################################################################

from obt import dep, path

###############################################################################

class pybind11(dep.StdProvider):
  name = "pybind11"
  def __init__(self):
    super().__init__(pybind11.name)
    self.declareDep("python")
    self.declareDep("cmake")
    PYTHON = dep.instance("python")
    self._builder = self.createBuilder(dep.CMakeBuilder)
    #self._builder.setCmVar("Python3_FIND_STRATEGY","LOCATION")
    #self._builder.setCmVar("Python3_ROOT_DIR",PYTHON.home_dir)
    self._builder.setCmVar("PYTHON_EXECUTABLE",PYTHON.executable)
    self._builder.requires(["python"])
    self._debug = True
    self._fetcher._debug = True
    self._builder._debug = True
  @property
  def _fetcher(self):
    return dep.GithubFetcher(name=pybind11.name,
                             repospec="tweakoz/pybind11",
                             revision="toz-2024-pep684",
                             recursive=True)
  def areRequiredSourceFilesPresent(self):
    return (self.source_root/"CMakeLists.txt").exists()
  def areRequiredBinaryFilesPresent(self):
    return (path.includes()/"pybind11"/"attr.h").exists()
